import * as tf from '@tensorflow/tfjs';
import { MobileNet, MobileNetMultiplier, OutputStride } from './mobilenet';
import { PersonSegmentationInput } from './types';
export declare class PersonSegmentation {
    mobileNet: MobileNet;
    constructor(mobileNet: MobileNet);
    predictForSegmentation(input: tf.Tensor3D, outputStride?: OutputStride): tf.Tensor3D;
    predictForPartMap(input: tf.Tensor3D, outputStride?: OutputStride): {
        segmentScores: tf.Tensor3D;
        partHeatmapScores: tf.Tensor3D;
    };
    estimatePersonSegmentation(input: PersonSegmentationInput, flipHorizontal?: boolean, outputStride?: OutputStride, segmentationThreshold?: number): Promise<Uint8Array>;
    estimatePartSegmentation(input: PersonSegmentationInput, flipHorizontal?: boolean, outputStride?: OutputStride, segmentationThreshold?: number): Promise<Int32Array>;
    dispose(): void;
}
export declare function load(multiplier?: MobileNetMultiplier): Promise<PersonSegmentation>;
export declare const mobilenetLoader: {
    load: (multiplier: MobileNetMultiplier) => Promise<MobileNet>;
};
